@@ -247,7 +247,7 @@ typedef struct {
    void         *video_planarmem;		/* opaque memory management information structure */
    CARD32       video_planarbuf; 		/* offset in video memory of planar YV12 buffer */
    
-#ifdef XF86DRI
+#ifdef SAVAGEDRI
    Bool         tried_agp;			/* TRUE if AGP allocation has been tried */
    CARD32	agpBase;			/* Physical address of aperture base */
    CARD32	agpBufferOffset;		/* Offset of buffer in AGP memory, or 0 if unavailable */
@@ -338,7 +338,7 @@ SavageClipVWindow(ScrnInfoPtr pScrn)
 
 void SavageInitVideo(ScreenPtr pScreen)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     XF86VideoAdaptorPtr *adaptors, *newAdaptors = NULL;
     XF86VideoAdaptorPtr newAdaptor = NULL;
     SavagePtr psav = SAVPTR(pScrn);
@@ -877,7 +877,7 @@ void SavageResetVideo(ScrnInfoPtr pScrn)
 static XF86VideoAdaptorPtr 
 SavageSetupImageVideo(ScreenPtr pScreen)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     SavagePtr psav = SAVPTR(pScrn);
     XF86VideoAdaptorPtr adapt;
     SavagePortPrivPtr pPriv;
@@ -1047,7 +1047,7 @@ SavageStopVideo(ScrnInfoPtr pScrn, pointer data, Bool shutdown)
       /*SavageClipVWindow(pScrn);*/
  	SavageStreamsOff( pScrn );
 
-#ifdef XF86DRI
+#ifdef SAVAGEDRI
 	if (pPriv->agpBufferMap != NULL) {
 	    SAVAGEDRIServerPrivatePtr pSAVAGEDRIServer = psav->DRIServerInfo;
 
@@ -1324,7 +1324,7 @@ SavageCopyPlanarData(
 static void
 SavageVideoSave(ScreenPtr pScreen, ExaOffscreenArea *area)
 {
-    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     SavagePtr psav = SAVPTR(pScrn);
     SavagePortPrivPtr pPriv = psav->adaptor->pPortPrivates[0].ptr;
 
@@ -1340,7 +1340,7 @@ SavageAllocateMemory(
     void **mem_struct,
     int size
 ){
-    ScreenPtr pScreen = screenInfo.screens[pScrn->scrnIndex];
+    ScreenPtr pScreen = xf86ScrnToScreen(pScrn);
     SavagePtr psav = SAVPTR(pScrn);
     int offset = 0;
 
@@ -1717,28 +1717,28 @@ SavageDisplayVideoNew(
         OUTREG(SEC_STREAM2_STRIDE_LPB, pitch & 0xfff );
         OUTREG(SEC_STREAM2_WINDOW_START, ((dstBox->x1+1) << 16) | (dstBox->y1+1) );
         OUTREG(SEC_STREAM2_WINDOW_SZ, ((dstBox->x2-dstBox->x1) << 16) 
-	   | (dstBox->x2-dstBox->x1) );
+	   | (dstBox->y2 - dstBox->y1) );
     } else if (psav->IsPrimary) {
         OUTREG(SEC_STREAM_FBUF_ADDR0, (offset + (x1>>15)) 
 	   & (0x7ffffff & ~BASE_PAD));
         OUTREG(SEC_STREAM_STRIDE, pitch & 0xfff );
         OUTREG(SEC_STREAM_WINDOW_START, ((dstBox->x1+1) << 16) | (dstBox->y1+1) );
         OUTREG(SEC_STREAM_WINDOW_SZ, ((dstBox->x2-dstBox->x1) << 16) 
-	   | (dstBox->x2-dstBox->x1) );
+	   | (dstBox->y2 - dstBox->y1) );
     } else {
         OUTREG(SEC_STREAM_FBUF_ADDR0, (offset + (x1>>15)) 
 	   & (0x7ffffff & ~BASE_PAD));
         OUTREG(SEC_STREAM_STRIDE, pitch & 0xfff );
         OUTREG(SEC_STREAM_WINDOW_START, ((dstBox->x1+1) << 16) | (dstBox->y1+1) );
         OUTREG(SEC_STREAM_WINDOW_SZ, ((dstBox->x2-dstBox->x1) << 16) 
-	   | (dstBox->x2-dstBox->x1) );
+	   | (dstBox->y2 - dstBox->y1) );
 #if 0
         OUTREG(SEC_STREAM2_FBUF_ADDR0, (offset + (x1>>15)) 
 	   & (0x7ffffff & ~BASE_PAD));
         OUTREG(SEC_STREAM2_STRIDE_LPB, pitch & 0xfff );
         OUTREG(SEC_STREAM2_WINDOW_START, ((dstBox->x1+1) << 16) | (dstBox->y1+1) );
         OUTREG(SEC_STREAM2_WINDOW_SZ, ((dstBox->x2-dstBox->x1) << 16) 
-	   | (dstBox->x2-dstBox->x1) );
+	   | (dstBox->y2 - dstBox->y1) );
 #endif
     }
 
@@ -1978,7 +1978,7 @@ SavagePutImage(
 
     /* Check whether AGP buffers can be allocated. If not, fall back to ordinary
        upload to framebuffer (slower) */
-#ifdef XF86DRI
+#ifdef SAVAGEDRI
     if (!pPriv->tried_agp && !psav->IsPCI && psav->drmFD > 0 && psav->DRIServerInfo != NULL) {
 	SAVAGEDRIServerPrivatePtr pSAVAGEDRIServer = psav->DRIServerInfo;
         
@@ -2014,7 +2014,7 @@ SavagePutImage(
 	    pPriv->agpBufferOffset = 0;
 	}
     }
-#endif /* XF86DRI */
+#endif /* SAVAGEDRI */
 
 
     /* Buffer for final packed frame */
@@ -2060,7 +2060,7 @@ SavagePutImage(
 	offsetV += tmp;
 	nlines = ((((y2 + 0xffff) >> 16) + 1) & ~1) - top;
         if (S3_SAVAGE4_SERIES(psav->Chipset) && psav->BCIforXv && (npixels & 0xF) == 0 && pPriv->video_planarbuf != 0) {
-#ifdef XF86DRI
+#ifdef SAVAGEDRI
             if (pPriv->agpBufferMap != NULL) {
 		/* Using copy to AGP memory */
 		SavageCopyPlanarDataBCI(
@@ -2073,7 +2073,7 @@ SavagePutImage(
 		    pPriv->agpBase + pPriv->agpBufferOffset,
 		    srcPitch, srcPitch2, dstPitch, nlines, npixels, TRUE);
             } else
-#endif /* XF86DRI */
+#endif /* SAVAGEDRI */
             {
 		/* Using ordinary copy to framebuffer */
 		SavageCopyPlanarDataBCI(
@@ -2359,7 +2359,7 @@ static void
 SavageInitOffscreenImages(ScreenPtr pScreen)
 {
     XF86OffscreenImagePtr offscreenImages;
-    SavagePtr psav = SAVPTR(xf86Screens[pScreen->myNum]);
+    SavagePtr psav = SAVPTR(xf86ScreenToScrn(pScreen));
 
     /* need to free this someplace */
     if (!psav->offscreenImages) {
